import pytest
from endpoints.api.tag_models_interface import RepositoryTagHistory, Tag
from mock import Mock

from data import model
from endpoints.api.tag_models_pre_oci import pre_oci_model

EMPTY_REPOSITORY = 'empty_repository'
EMPTY_NAMESPACE = 'empty_namespace'
BAD_REPOSITORY_NAME = 'bad_repository_name'
BAD_NAMESPACE_NAME = 'bad_namespace_name'


@pytest.fixture
def get_monkeypatch(monkeypatch):
  return monkeypatch


def mock_out_get_repository(monkeypatch, namespace_name, repository_name):
  def return_none(namespace_name, repository_name):
    return None

  def return_repository(namespace_name, repository_name):
    return 'repository'

  if namespace_name == BAD_NAMESPACE_NAME or repository_name == BAD_REPOSITORY_NAME:
    return_function = return_none
  else:
    return_function = return_repository

  monkeypatch.setattr(model.repository, 'get_repository', return_function)


def create_mock_tag(name, reversion, lifetime_start_ts, lifetime_end_ts, mock_id, docker_image_id,
                    manifest_list):
  tag_mock = Mock()
  tag_mock.name = name
  image_mock = Mock()
  image_mock.docker_image_id = docker_image_id
  tag_mock.image = image_mock
  tag_mock.reversion = reversion
  tag_mock.lifetime_start_ts = lifetime_start_ts
  tag_mock.lifetime_end_ts = lifetime_end_ts
  tag_mock.id = mock_id
  tag_mock.manifest_list = manifest_list
  tag = Tag(name=name, reversion=reversion, image=image_mock, docker_image_id=docker_image_id,
            lifetime_start_ts=lifetime_start_ts, lifetime_end_ts=lifetime_end_ts,
            manifest_list=manifest_list)
  return tag_mock, tag


first_mock, first_tag = create_mock_tag('tag1', 'rev1', 'start1', 'end1', 'id1',
                                        'docker_image_id1', [])
second_mock, second_tag = create_mock_tag('tag2', 'rev2', 'start2', 'end2', 'id2',
                                          'docker_image_id2', ['manifest'])


def mock_out_list_repository_tag_history(monkeypatch, namespace_name, repository_name, page, size,
                                         specific_tag):
  def list_empty_tag_history(repository, page, size, specific_tag):
    return [], {}, False

  def list_filled_tag_history(repository, page, size, specific_tag):
    tags = [first_mock, second_mock]
    return tags, {
      first_mock.id: first_mock.manifest_list,
      second_mock.id: second_mock.manifest_list
    }, len(tags) > size

  def list_only_second_tag(repository, page, size, specific_tag):
    tags = [second_mock]
    return tags, {second_mock.id: second_mock.manifest_list}, len(tags) > size

  if namespace_name == EMPTY_NAMESPACE or repository_name == EMPTY_REPOSITORY:
    return_function = list_empty_tag_history
  else:
    if specific_tag == 'tag2':
      return_function = list_only_second_tag
    else:
      return_function = list_filled_tag_history

  monkeypatch.setattr(model.tag, 'list_repository_tag_history', return_function)


@pytest.mark.parametrize(
  'expected, namespace_name,        repository_name,       page,  size, specific_tag', [
    (None, BAD_NAMESPACE_NAME, 'repository_name', 1, 100, None),
    (None, 'namespace_name', BAD_REPOSITORY_NAME, 1, 100, None),
    (RepositoryTagHistory(tags=[], more=False), EMPTY_NAMESPACE, EMPTY_REPOSITORY, 1, 100, None),
    (RepositoryTagHistory(tags=[first_tag, second_tag], more=False), 'namespace', 'repository', 1,
     100, None),
    (RepositoryTagHistory(tags=[first_tag, second_tag], more=True), 'namespace', 'repository', 1,
     1, None),
    (RepositoryTagHistory(tags=[second_tag], more=False), 'namespace', 'repository', 1, 100,
     'tag2'),
  ])
def test_list_repository_tag_history(expected, namespace_name, repository_name, page, size,
                                     specific_tag, get_monkeypatch):
  mock_out_get_repository(get_monkeypatch, namespace_name, repository_name)
  mock_out_list_repository_tag_history(get_monkeypatch, namespace_name, repository_name, page,
                                       size, specific_tag)
  assert pre_oci_model.list_repository_tag_history(namespace_name, repository_name, page, size,
                                                   specific_tag) == expected
